//***************************************************************************//
//*                                                                         *//
//* Unit:    drj_Codec                                                      *//
//* Author:  Dark Magus                                                     *//
//* Date:    17.09.01                                                       *//
//*                                                                         *//
//*    /      *//
//*  Dream Journal (v. 2.0).                                         *//
//*                                                                         *//
//***************************************************************************//
unit drj_Codec;

//-[ INTERFACE ]---------------------------------------------------------------
interface

const
  NOF_SYMBOLS = 130;
  NOF_STRINGS = 5;

type TCodingArray = Array[1..NOF_STRINGS] of String;

const
  ORDINARY_STRING : String = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz 1234567890_Ũ';

const
  CODING_ARRAY : TCodingArray = ('zyxwvutsrqponmlkjihgfedcbaZYXWVUTSRQPONMLKJIHGFEDCBA_ƨ6574839201 ',
                                 'badcfehgjilknmporqtsvuxwzyBADCFEHGJILKNMPORQTSVUXWZY_ 0192837465ƨ',
                                 'qwertyuiopasdfghjklzxcvbnmMNBVCXZLKJHGFDSAPOIUYTREWQƨ_ 0987654321',
                                 'qazwsxedcrfvtgbyhnujmikolpPLOKMIJNUHBYGVTFCRDXESZWAQ0987654321 _ƨ',
                                 'yturieowpqhgjfkdlsavbcnxmzQWSAZXCDERFVBGTYUJHNMKLIOP1234567890_ ƨ');


type TCodec = class (TObject)
  private

    function Ord (iSymbol : Char) : Integer;
    function Symbol (iOrd : Integer) : Char;

  public
    constructor Create;
    destructor Destroy; override;

    function Encode (const iStr : String) : String;
    function Decode (const iStr : String) : String;

end;

//-[ IMPLEMENTATION ]----------------------------------------------------------
implementation

constructor TCodec.Create;
begin
  inherited Create;
end;

destructor TCodec.Destroy;
begin
  inherited Destroy;
end;

function TCodec.Encode (const iStr : String) : String;
var
  i     : Integer;
  index : Integer;
begin
  for i := 1 to Length (iStr) do
  begin
    index := (i mod NOF_STRINGS) + 1;
    Result := Result + CODING_ARRAY[index][Ord (iStr[i])];
  end;
end;

function TCodec.Decode (const iStr : String) : String;
var
  i     : Integer;
  index : Integer;
begin
  for i := 1 to Length (iStr) do
  begin
    index := (i mod NOF_STRINGS) + 1;
    Result := Result + Symbol (Pos (iStr[i], CODING_ARRAY[index]));
  end;
end;

function TCodec.Ord (iSymbol : Char) : Integer;
begin
  Result := Pos (iSymbol, ORDINARY_STRING);
end;

function TCodec.Symbol (iOrd : Integer) : Char;
begin
  Result := #0;
  if ((iOrd > 0) and (iOrd <= NOF_SYMBOLS)) then
  begin
    Result := ORDINARY_STRING[iOrd];
  end
end;

//-[ END ]---------------------------------------------------------------------
end.
