//***************************************************************************//
//*                                                                         *//
//* Unit:    drj_UserForm                                                   *//
//* Author:  Dark Magus                                                     *//
//* Date:    28.12.01                                                       *//
//*                                                                         *//
//*          - *//
//*     (  Dream Journal "Wi-  *//
//* zardry").                                                               *//
//*                                                                         *//
//***************************************************************************//
unit drj_UserForm;

//-[ INTERFACE ]---------------------------------------------------------------
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls,

  drj_Resources,

  xml_Node,
  xml_Types;

type Tfrm_DreamProperties = class (TForm)
    pnl_WorkArea : TPanel;
    procedure CloseForm (Sender : TObject);
    procedure ClickOk (Sender : TObject);
    procedure ClickCancel (Sender : TObject);

  private
    function BooleanToString (Bool : Boolean) : String;

  public
    procedure FillIn (xmlNode : TXMLNode; iContainer : TWinControl);
    procedure SaveInfo (xmlNode : TXMLNode; iContainer : TWinControl);
    procedure ClearControls (iContainer : TWinControl);

end;

//-[ IMPLEMENTATION ]----------------------------------------------------------
implementation

{$R *.DFM}

procedure Tfrm_DreamProperties.CloseForm (Sender : TObject);
begin
  ModalResult := mrOk;
end;

procedure Tfrm_DreamProperties.ClickOk (Sender : TObject);
begin
  ModalResult := mrOk;
end;

procedure Tfrm_DreamProperties.ClickCancel (Sender : TObject);
begin
  ModalResult := mrCancel;
end;

function Tfrm_DreamProperties.BooleanToString (Bool : Boolean) : String;
begin
  if (Bool) then
  begin
    Result := TRUE_VALUE;
  end
  else
  begin
    Result := FALSE_VALUE;
  end;
end;

procedure Tfrm_DreamProperties.FillIn (xmlNode : TXMLNode; iContainer : TWinControl);
var
  i             : Integer;
  j             : Integer;
  k             : Integer;
  component     : String;
  value         : String;
  control       : TControl;
  comboBox      : TComboBox;
  childNode     : TXMLNode;
begin
  if (xmlNode <> nil) then
  begin
    if (xmlNode.TagType = ttNormal) then
    begin
      for i := 0 to xmlNode.GetNumberOfChildren - 1 do
      begin
        childNode := xmlNode.GetChild (i);
        component := childNode.GetAttributeValue (ATTRIBUTE_CONTROL);
        value := childNode.GetAttributeValue (ATTRIBUTE_VALUE);
        for j := 0 to iContainer.ControlCount - 1 do
        begin
          control := iContainer.Controls[j];
          if (control.Name = component) then
          begin
            if (control.ClassName = 'TEdit') then
            begin
              (control as TEdit).Text := value;
              Break;
            end;
            if (control.ClassName = 'TMemo') then
            begin
              (control as TMemo).Text := value;
              Break;
            end;
            if (control.ClassName = 'TComboBox') then
            begin
              comboBox := (control as TComboBox);
              for k := 0 to comboBox.Items.Count - 1 do
              begin
                if (value = comboBox.Items[k]) then
                begin
                  comboBox.ItemIndex := k;
                  Break;
                end;
              end;
              Break;
            end;
            if (control.ClassName = 'TCheckBox') then
            begin
              (control as TCheckBox).Checked := (value = TRUE_VALUE);
              Break;
            end;
            if (control.ClassName = 'TRadioButton') then
            begin
              (control as TRadioButton).Checked := (value = TRUE_VALUE);
              Break;
            end;
            if (control.ClassName = 'TGroupBox') then
            begin
              FillIn (childNode, (control as TGroupBox));
            end;
          end;
        end;
      end;
    end;
  end;
end;

procedure Tfrm_DreamProperties.SaveInfo (xmlNode : TXMLNode; iContainer : TWinControl);
var
  i         : Integer;
  index     : Integer;
  control   : TControl;
  childNode : TXMLNode;
begin
  if (xmlNode <> nil) then
  begin
    if (xmlNode.TagType <> ttNormal) then
    begin
      xmlNode.TagType := ttNormal;
    end;
    index := 0;
    for i := 0 to iContainer.ControlCount - 1 do
    begin
      control := iContainer.Controls[i];
      childNode := xmlNode.GetChild (index);
      if (childNode = nil) then
      begin
        childNode := TXMLNode.Create;
        childNode.TagName := TAG_PARAMETER;
        childNode.TagType := ttShorted;
        xmlNode.AddChildAsObject (childNode);
        childNode.Level := xmlNode.Level + 1;
      end;

      if (control.ClassName = 'TEdit') then
      begin
        childNode.SetAttributeValue (ATTRIBUTE_CONTROL, control.Name);
        childNode.SetAttributeValue (ATTRIBUTE_VALUE, (control as TEdit).Text);
        index := index + 1;
        Continue;
      end;
      if (control.ClassName = 'TMemo') then
      begin
        childNode.SetAttributeValue (ATTRIBUTE_CONTROL, control.Name);
        childNode.SetAttributeValue (ATTRIBUTE_VALUE, (control as TMemo).Text);
        index := index + 1;
        Continue;
      end;
      if (control.ClassName = 'TComboBox') then
      begin
        childNode.SetAttributeValue (ATTRIBUTE_CONTROL, control.Name);
        childNode.SetAttributeValue (ATTRIBUTE_VALUE, (control as TComboBox).Text);
        index := index + 1;
        Continue;
      end;
      if (control.ClassName = 'TCheckBox') then
      begin
        childNode.SetAttributeValue (ATTRIBUTE_CONTROL, control.Name);
        childNode.SetAttributeValue (ATTRIBUTE_VALUE, BooleanToString ((control as TCheckBox).Checked));
        index := index + 1;
        Continue;
      end;
      if (control.ClassName = 'TRadioButton') then
      begin
        childNode.SetAttributeValue (ATTRIBUTE_CONTROL, control.Name);
        childNode.SetAttributeValue (ATTRIBUTE_VALUE, BooleanToString ((control as TRadioButton).Checked));
        index := index + 1;
        Continue;
      end;
      if (control.ClassName = 'TGroupBox') then
      begin
        childNode.SetAttributeValue (ATTRIBUTE_CONTROL, control.Name);
        childNode.TagType := ttNormal;
        SaveInfo (childNode, (control as TGroupBox));
        index := index + 1;
        Continue;
      end;

      if ((control.ClassName <> 'TEdit')
        and (control.ClassName <> 'TMemo')
        and (control.ClassName <> 'TComboBox')
        and (control.ClassName <> 'TCheckBox')
        and (control.ClassName <> 'TRadioButton')
        and (control.ClassName <> 'TGroupBox')) then
      begin
        xmlNode.DeleteChild (index);
      end;
    end;
  end;
end;

procedure Tfrm_DreamProperties.ClearControls (iContainer : TWinControl);
var
  i       : Integer;
  control : TControl;
begin
  for i := 0 to iContainer.ControlCount - 1 do
  begin
    control := iContainer.Controls[i];
    if (control.ClassName = 'TEdit') then
    begin
      (control as TEdit).Text := '';
      Continue;
    end;
    if (control.ClassName = 'TMemo') then
    begin
      (control as TMemo).Text := '';
      Continue;
    end;
    if (control.ClassName = 'TComboBox') then
    begin
      (control as TComboBox).Text := '';
      (control as TComboBox).ItemIndex := -1;
      Continue;
    end;
    if (control.ClassName = 'TCheckBox') then
    begin
      (control as TCheckBox).Checked := False;
      Continue;
    end;
    if (control.ClassName = 'TRadioButton') then
    begin
      (control as TRadioButton).Checked := False;
      Continue;
    end;
    if (control.ClassName = 'TGroupBox') then
    begin
      ClearControls (control as TGroupBox);
      Continue;
    end;
  end;
end;

//-[ END ]---------------------------------------------------------------------
end.
